/*
 * puzzle.h
 *
 *
 * declarations for puzzle.c
 *
 * by Korbinian Strimmer and Arndt von Haeseler (1995)
 * this file belongs to the Puzzle version 2.2 distribution
 *
 */


/* load all necessary libraries */
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

/* all EXTERNal variables are local in puzzle.c */
#ifdef MAIN_MODULE
#define EXTERN
#else
#define EXTERN extern
#endif

/* global definitions */
#define TRUE 1
#define FALSE 0
#define VERSION    "2.2"
#define DATE       "10 October 1995"

/* declarations of other modules */
#include "qml.h"
#include "matrixut.h"


/* tree functions */

/* type definitions */
typedef struct oneedge {
	/* pointer to other three edges */
	struct oneedge *up;
	struct oneedge *downleft;
	struct oneedge *downright;
	int numedge;  /* number of edge */
	int edgeinfo; /* info about edge - in puzzle an integer */
	int *edgemap; /* pointer to the local edgemap (= integer vector) */
} ONEEDGE;

/* variables */
EXTERN ONEEDGE *edge;    /* vector with all the edges of the tree */
EXTERN int *edgeofleaf;  /* vector with edge number of all leaves */
EXTERN int nextedge;     /* number of edges in the current tree */
EXTERN int nextleaf;     /* next leaf to add to tree */
/* = number of leaves in the current tree (leaves/edges are counted beginning with 0) */
EXTERN int minedge;      /* edge with minimum edgeinfo */
EXTERN int mininfo;      /* value of edgeinfo on that edge */
EXTERN int column;       /* for breaking lines in writing tree to treefile */
EXTERN int cuttingedge;  /* cutting edge ("Use selected quartets") */
EXTERN ivector edgelist; /* list of edges in cluster (fast Puzzle) */
EXTERN ivector leaflist; /* list of leaves in cluster (fast Puzzle) */


/* quartet management */

/* variables */
EXTERN int chooseA;
EXTERN int chooseB;
EXTERN int chooseC;
EXTERN double d1;
EXTERN double d2;
EXTERN double d3;
EXTERN int lowestml;
EXTERN int startA;
EXTERN int startB;
EXTERN int startC;
EXTERN int startD;
EXTERN unsigned char *quartetinfo;


/* random numbers */

/* necessary libraries */
#include <stdlib.h>
#include <time.h>
#include <math.h>

/* definitions */
#define IM1 2147483563
#define IM2 2147483399
#define AM (1.0/IM1)
#define IMM1 (IM1-1)
#define IA1 40014
#define IA2 40692
#define IQ1 53668
#define IQ2 52774
#define IR1 12211
#define IR2 3791
#define NTAB 32
#define NDIV (1+IMM1/NTAB)
#define EPS 1.2e-7
#define RNMX (1.0-EPS)

/* variables */
EXTERN long idum;


/* quartet puzzling */

/* pointer to files */
EXTERN FILE *treefp; /* treefile */

/* variables */
EXTERN int Info_optn;
EXTERN int Approx_optn;
EXTERN int pmode_optn;
EXTERN int save_optn;
EXTERN int Numtrial;       /* number of Puzzle trees */
EXTERN int Currtrial;      /* counter for Puzzle trees */
EXTERN int numleaves;      /* number of leaves remaining (fast Puzzle) */
EXTERN ivector trueID;     /* leaf -> taxon on this leaf */
EXTERN ivector freeID;
EXTERN ivector freeleaves; /* list of free leaves (fast Puzzle): 1 = taxon available */
